////////////////////////////////////////////////////////////////////////////

SOUND* impact_wav = "impact.wav";

////////////////////////////////////////////////////////////////////////////

STRING* bullet_mdl = "bullet.mdl";

////////////////////////////////////////////////////////////////////////////

function fire_bullets(); // creates the bullets
function remove_bullets(); // removes the bullets after they've hit something
function got_shot();
function move_enemy_bullets();

////////////////////////////////////////////////////////////////////////////

#define idle 1
#define attacking 2
#define dead 3
#define status skill1

////////////////////////////////////////////////////////////////////////////

function remove_bullets() // this function runs when the bullet collides with something
{
	my.event = NULL; // don't allow a bullet to trigger several events
	wait (1);
	set (my, PASSABLE);
	set (my, INVISIBLE);
	ent_playsound (my, impact_wav, 500);
	wait (-0.2);
	ent_remove (my);

}

action enemy() // attach this action to your enemies
{
	VECTOR temp[3];
	VECTOR temporary[3];
	var idle_percentage = 0;
	var run_percentage = 0;
	var attack_percentage = 0;
	var death_percentage = 0;
	var right_once = 0;
	var left_once = 0;
	var idle_time = 0;
	set (my, POLYGON); // use accurate collision detection
	my.health = 100;
	my.emask = ENABLE_IMPACT | ENABLE_ENTITY; // the enemy is sensitive to impact with player's bullets
	my.event = got_shot; // and runs this function when it is hit
	my.status = idle; // that's the same thing with my.skill1 = 1; (really!)
	ent_create ("health.pcx", nullvector, health_indicator);
	while (my.status != dead) // this loop will run for as long as my.skill1 isn't equal to 3
	{
		vec_set(temporary.x, my.x);
		temporary.z -= 10000;
		my.z -= c_trace(my.x, temporary.x, IGNORE_ME | USE_BOX | IGNORE_PASSABLE) - 20; // play with 40

		if (my.status == idle) // hanging around?
		{
			ent_animate(my, "stand", idle_percentage, ANM_CYCLE); // play the "stand" aka idle animation
			idle_percentage += 3 * time_step; // "3" controls the animation speed
			if (vec_dist (player.x, my.x) < 1000) // the player has come too close?
			{
				// scanned in the direction of the pan angle and detected the player?
				if ((c_scan(my.x, my.pan, vector(120, 60, 1000), IGNORE_ME) > 0) && (you == player))
				{
					my.status = attacking; // then attack the player even if it hasn't fired at the enemy yet
				}
			}
		}
		if (my.status == attacking) // shooting at the player?
		{
			if (player.health > 0) 
			{
				vec_set(temp, player.x); 
				vec_sub(temp,my.x); 
				vec_to_angle(my.pan, temp); // turn the enemy towards the player
				if (vec_dist (player.x, my.x) > 500) // the player is away? Then run towards him / her
				{
					c_move (my, vector(10 * time_step, 0, 0), nullvector, IGNORE_PASSABLE | GLIDE);
					ent_animate(my, "run", run_percentage, ANM_CYCLE); // play the "run" animation
					run_percentage += 6 * time_step; // "6" controls the animation speed
				}
				else // the player is close here, so attack him / her
				{
					ent_animate(my, "attack", attack_percentage, ANM_CYCLE); // play the "attack" animation
					attack_percentage += 6 * time_step;
					attack_percentage %= 100;
				}
				if ((attack_percentage > 25) && (attack_percentage < 50) && (right_once == 0))
				{
					right_once = 1;
					vec_for_vertex (temp, my, 2);
					// create the projectile at enemy's right fist position and attach it the "move_enemy_bullets" function
					ent_create (bullet_mdl, temp, move_enemy_bullets); 
				}
				if ((attack_percentage > 70) && (attack_percentage < 80) && (left_once == 0))
				{
					left_once = 1;
					vec_for_vertex (temp, my, 139);
					// create the projectile at enemy's left fist position and attach it the "move_enemy_bullets" function
					ent_create (bullet_mdl, temp, move_enemy_bullets); 
				}
				if (attack_percentage > 85) // we are ready to reset the left and right arms here
				{
					right_once = 0;	
					left_once = 0;
				}
				if (vec_dist (player.x, my.x) > 1000) // the player has moved far away from the enemy?
				{
					idle_time += time_step / 16;
					if (idle_time > 5) // the enemy has run towards the player for 5 seconds without being able to see it?
						my.status = idle; // then the player must be far away, so switch to "idle"
				}
				else
				{
					idle_time = 0; // reset idle_time
				}
			}
			else // the player is dead?
			{
					my.status = idle; // then switch to "idle"		
			}
		}
		wait (1);
	}
	if (my.health < 0) {my.health = 0;}
	while (death_percentage < 100) // the loop runs until the "death" animation percentage reaches 100%	
	{
		ent_animate(my, "death", death_percentage, NULL); // play the "die" animation only once
		death_percentage += 3 * time_step; // "3" controls the animation speed
		wait (1);
	}
	set (my, PASSABLE); // allow the player to pass through the corpse now
}

function got_shot()
{
	if (you.skill30 != 2) {return;} // didn't collide with one of player's bullets? Then nothing should happen
	my.health -= 5; // the enemy was hit by one of player's bullets here
	if (my.health <= 0) // dead?
	{
		my.status = dead; // stop the loop enemy's action
		my.event = NULL; // the enemy is dead, so it should stop reacting to other bullets from now on
		return; // end this function here
	}
	else // got shot but not (yet) dead?
	{
		my.status = attacking; // same thing with my.skill1 = 2
	}
}

function move_enemy_bullets()
{
	VECTOR bullet_speed[3]; // this var will store the speed of the bullet
	my.skill30 = 1; // I'm a bullet
	my.emask = ENABLE_IMPACT | ENABLE_ENTITY | ENABLE_BLOCK;
	my.event = remove_bullets; // when it collides with something, its event function (remove_bullets) will run
	my.pan = you.pan; // the bullet has the same pan
	my.tilt = you.tilt; // and tilt with the enemy
	bullet_speed.x = 50 * time_step; // adjust the speed of the bullet here
	bullet_speed.y = 0; // the bullet doesn't move sideways
	bullet_speed.z = 0; // or up / down on the z axis
	// the loop will run for as long as the bullet exists (it isn't "null")
	while (my) 
	{
		bullet_particles();
		// move the bullet ignoring its creator (the enemy) and the passable entities
		c_move (my, bullet_speed, nullvector, IGNORE_YOU | IGNORE_PASSABLE); 
		wait (1);
	}
}
